==> org-lsst-ccs-drivers-reb-2.8.2-sources.jar: org/lsst/ccs/drivers/reb/Daq4Client.java <==
package org.lsst.ccs.drivers.reb;

import org.lsst.ccs.drivers.commons.DriverException;

/**
 *  Java interfaces to REBs connected via the DAQ V4 system.
 *
 *  @author Owen Saxton
 */
public class Daq4Client {

    static void loadLibrary() {
        System.loadLibrary("RebDaq4");
    }


    /**
     *  Register interface.
     */
    public static class Registers implements RegClient.Impl {

        private static final int MAX_RETRY = 2;

        Registers() {
            loadLibrary();
        }

        @Override
        public native long newRegClient(int id, String part);

        @Override
        public native void deleteRegClient(long handle);

        @Override
        public int readReg(long handle, int address) throws REBException {
            int[] values = new int[1];
            readRegs(handle, address, values, 0, 1);
            return values[0];
        }

        @Override
        public void readRegs(long handle, int address, int[] values, int offset, int count) throws REBException {
            for (int j = 0; j <= MAX_RETRY; j++) {
                try {
                    readRegs1(handle, address, values, offset, count);
                    return;
                }
                catch (DriverException e) {
                    if (j >= MAX_RETRY) {
                        throw e;
                    }
                }
            }
        }

        public native void readRegs1(long handle, int address, int[] values, int offset, int count) throws REBException;

        @Override
        public void writeReg(long handle, int address, int value) throws REBException {
            int[] values = {value};
            writeRegs(handle, address, values, 0, 1);
        }

        @Override
        public void writeRegs(long handle, int address, int[] values, int offset, int count) throws REBException {
            for (int j = 0; j <= MAX_RETRY; j++) {
                try {
                    writeRegs1(handle, address, values, offset, count);
                    return;
                }
                catch (DriverException e) {
                    if (j >= MAX_RETRY) {
                        throw e;
                    }
                }
            }
        }

        public native void writeRegs1(long handle, int address, int[] values, int offset, int count) throws REBException;

        @Override
        public int updateReg(long handle, int address, int mask, int value) throws REBException {
            int[] values = {0};
            readRegs(handle, address, values, 0, 1);
            int oldVal = values[0];
            values[0] = (values[0] & ~mask) | (value & mask);
            writeRegs(handle, address, values, 0, 1);
            return oldVal;
        }

        @Override
        public native void reset(long handle, int type) throws REBException;

    }


    /**
     *  Image interface.
     */
    public static class Images implements ImageClient.Impl  {

        long handle;

        Images() {
            loadLibrary();
            initSys();
        }

        private native static void initSys();

        @Override
        public native void newImageClient(int id, String part);

        @Override
        public native void deleteImageClient();

        @Override
        public native Image waitForImage(Image image);
        
        @Override
        public native boolean getImage(Image image);

        @Override
        public native void interrupt();

    }


    /**
     *  Global interface.
     */
    public static class Global implements GlobalClient.Impl  {

        long handle;

        Global() {
            loadLibrary();
            initSys();
        }

        private native static void initSys();

        @Override
        public void newGlobalClient(int[] ids, String[] ifc) {
            handle = newGlobalClient1(ifc[0]);
        }

        @Override
        public void newGlobalClient(String part) {
            handle = newGlobalClient1(part);
        }

        private native long newGlobalClient1(String part);

        @Override
        public void deleteGlobalClient() {
            deleteGlobalClient1(handle);
            handle = 0;
        }

        private native void deleteGlobalClient1(long handle);

        @Override
        public int getHwType() {
            return RegClient.HDW_TYPE_DAQ4;
        }

        @Override
        public void setRegisterList(int rebType, int[] registers) {
            setRegisterList1(handle, rebType, registers);
        }

        private native void setRegisterList1(long handle, int rebType, int[] registers);

        @Override
        public long triggerImage(String name) {
            return triggerImage(name, "", 1, "");
        }

        @Override
        public long triggerImage(String name, String folder, int opcode, String annotation, int... ids) {
            return triggerImage1(handle, name, folder, opcode, annotation, ids);
        }

        private native long triggerImage1(long handle, String name, String folder, int opcode, String annotation,
                                          int... ids);

        @Override
        public long startSequencer(int opcode) {
            return startSequencer1(handle, opcode);
        }

        private native long startSequencer1(long handle, int opcode);

        private native void setRebs1(long handle, int[] ids);

        private native void addRebs1(long handle, int[] ids);

        private native void removeRebs1(long handle, int[] ids);

        private native void resetRebs1(long handle);

    }

}

==> org-lsst-ccs-drivers-reb-2.10.1-sources.jar: org/lsst/ccs/drivers/reb/Daq4Client.java <==
package org.lsst.ccs.drivers.reb;

import java.util.logging.Level;
import java.util.logging.Logger;
import org.lsst.ccs.drivers.commons.DriverException;

/**
 *  Java interfaces to REBs connected via the DAQ V4 system.
 *
 *  @author Owen Saxton
 */
public class Daq4Client {

    static void loadLibrary() {
        System.loadLibrary("RebDaq4");
    }


    /**
     *  Register interface.
     */
    public static class Registers implements RegClient.Impl {

        private static final int MAX_RETRY = 2;
        private static final Logger LOG = Logger.getLogger(Registers.class.getName());

        Registers() {
            loadLibrary();
        }

        @Override
        public native long newRegClient(int id, String part);

        @Override
        public native void deleteRegClient(long handle);

        @Override
        public int readReg(long handle, int address) throws REBException {
            int[] values = new int[1];
            readRegs(handle, address, values, 0, 1);
            return values[0];
        }

        @Override
        public void readRegs(long handle, final int address, int[] values, int offset, int count) throws REBException {
            final long startNanos = System.nanoTime();
            for (int j = 0; j <= MAX_RETRY; j++) {
                try {
                    readRegs1(handle, address, values, offset, count);
                    if (j > 0) {
                        int finalJ = j;
                        LOG.log(Level.WARNING, () -> String.format("Reading register @%x required %d retries, and %,dns", address, finalJ, System.nanoTime() - startNanos));
                    }
                    return;
                }
                catch (DriverException e) {
                    if (j >= MAX_RETRY) {
                        throw e;
                    }
                }
            }
        }

        public native void readRegs1(long handle, int address, int[] values, int offset, int count) throws REBException;

        @Override
        public void writeReg(long handle, int address, int value) throws REBException {
            int[] values = {value};
            writeRegs(handle, address, values, 0, 1);
        }

        @Override
        public void writeRegs(long handle, int address, int[] values, int offset, int count) throws REBException {
            for (int j = 0; j <= MAX_RETRY; j++) {
                try {
                    writeRegs1(handle, address, values, offset, count);
                    return;
                }
                catch (DriverException e) {
                    if (j >= MAX_RETRY) {
                        throw e;
                    }
                }
            }
        }

        public native void writeRegs1(long handle, int address, int[] values, int offset, int count) throws REBException;

        @Override
        public int updateReg(long handle, int address, int mask, int value) throws REBException {
            int[] values = {0};
            readRegs(handle, address, values, 0, 1);
            int oldVal = values[0];
            values[0] = (values[0] & ~mask) | (value & mask);
            writeRegs(handle, address, values, 0, 1);
            return oldVal;
        }

        @Override
        public native void reset(long handle, int type) throws REBException;

    }


    /**
     *  Image interface.
     */
    public static class Images implements ImageClient.Impl  {

        long handle;

        Images() {
            loadLibrary();
            initSys();
        }

        private native static void initSys();

        @Override
        public native void newImageClient(int id, String part);

        @Override
        public native void deleteImageClient();

        @Override
        public native Image waitForImage(Image image);
        
        @Override
        public native boolean getImage(Image image);

        @Override
        public native void interrupt();

    }


    /**
     *  Global interface.
     */
    public static class Global implements GlobalClient.Impl  {

        long handle;

        Global() {
            loadLibrary();
            initSys();
        }

        private native static void initSys();

        @Override
        public void newGlobalClient(int[] ids, String[] ifc) {
            handle = newGlobalClient1(ifc[0]);
        }

        @Override
        public void newGlobalClient(String part) {
            handle = newGlobalClient1(part);
        }

        private native long newGlobalClient1(String part);

        @Override
        public void deleteGlobalClient() {
            deleteGlobalClient1(handle);
            handle = 0;
        }

        private native void deleteGlobalClient1(long handle);

        @Override
        public int getHwType() {
            return RegClient.HDW_TYPE_DAQ4;
        }

        @Override
        public void setRegisterList(int rebType, int[] registers) {
            setRegisterList1(handle, rebType, registers);
        }

        private native void setRegisterList1(long handle, int rebType, int[] registers);

        @Override
        public long triggerImage(String name) {
            return triggerImage(name, "", 1, "");
        }

        @Override
        public long triggerImage(String name, String folder, int opcode, String annotation, int... ids) {
            return triggerImage1(handle, name, folder, opcode, annotation, ids);
        }

        private native long triggerImage1(long handle, String name, String folder, int opcode, String annotation,
                                          int... ids);

        @Override
        public long startSequencer(int opcode) {
            return startSequencer1(handle, opcode);
        }

        private native long startSequencer1(long handle, int opcode);

        private native void setRebs1(long handle, int[] ids);

        private native void addRebs1(long handle, int[] ids);

        private native void removeRebs1(long handle, int[] ids);

        private native void resetRebs1(long handle);

    }

}
